#include <SoftwareSerial.h>
#include <Wire.h>
#include "Kalman.h"

SoftwareSerial bluetooth(3, 2); // TX = 3; RX = 2; SDA = A4, orange; SCL = A5, green
Kalman kalmanX;
Kalman kalmanY;

#define pinTrig 4
#define pinEcho 5
#define pinLight A0

unsigned int impulseTime = 0;
uint8_t gyroAddress_1 = 0x68;
uint8_t gyroAddress_2 = 0x69;

uint8_t* data;
int16_t accX;
int16_t accY;
int16_t accZ;
int16_t tempRaw;
int16_t gyroX;
int16_t gyroY;
int16_t gyroZ;
double accXangle;
double accYangle;
double temp;
double gyroXangle = 180;
double gyroYangle = 180;
double compAngleX = 180;
double compAngleY = 180;
double kalAngleX;
double kalAngleY;
double gyroXrate;
double gyroYrate;
uint32_t timer;

void setup() {
  pinMode(pinTrig, OUTPUT);
  pinMode(pinEcho, INPUT);
  bluetooth.begin(9600);
  Wire.begin();
  i2cWrite(gyroAddress_1, 0x6B, 0x00);
  i2cWrite(gyroAddress_2, 0x6B, 0x00);
  kalmanX.setAngle(180);
  kalmanY.setAngle(180);
  timer = micros();

  Serial.begin(9600);
}

void loop() {
  digitalWrite(pinTrig, HIGH);                    //     trig 
  delayMicroseconds(10);                          //  10 
  digitalWrite(pinTrig, LOW);                     // 
  bluetooth.print("D");
  bluetooth.print(pulseIn(pinEcho, HIGH) / 58);   //   ,   

  bluetooth.print("L");
  bluetooth.print(analogRead(pinLight));          // 

  data = i2cRead(gyroAddress_1 , 0x3B, 14);       //  1
  gyroscope();
  bluetooth.print("X");
  bluetooth.print(kalAngleX);
  bluetooth.print("Y");
  bluetooth.print(kalAngleY);
  
  Serial.print("X");
  Serial.print(kalAngleX);
  Serial.print("Y");
  Serial.println(kalAngleY);
  
  data = i2cRead(gyroAddress_2 ,0x3B, 14);        //  2
  gyroscope();
  bluetooth.print("X");
  bluetooth.print(kalAngleX);
  bluetooth.print("Y");
  bluetooth.println(kalAngleY);

  Serial.print("X");
  Serial.print(kalAngleX);
  Serial.print("Y");
  Serial.println(kalAngleY);
  delay(500);
}

void gyroscope() {
  accX = ((data[0] << 8) | data[1]);
  accY = ((data[2] << 8) | data[3]);
  accZ = ((data[4] << 8) | data[5]);
  tempRaw = ((data[6] << 8) | data[7]);
  gyroX = ((data[8] << 8) | data[9]);
  gyroY = ((data[10] << 8) | data[11]);
  gyroZ = ((data[12] << 8) | data[13]);
  accYangle = (atan2(accX, accZ) + PI) * RAD_TO_DEG;
  accXangle = (atan2(accY, accZ) + PI) * RAD_TO_DEG;
  gyroXrate = (double)gyroX / 131.0;
  gyroYrate = -((double)gyroY / 131.0);
  gyroXangle += kalmanX.getRate() * ((double)(micros() - timer) / 1000000); 
  gyroYangle += kalmanY.getRate() * ((double)(micros() - timer) / 1000000);
  kalAngleX = kalmanX.getAngle(accXangle, gyroXrate, (double)(micros() - timer) / 1000000); 
  kalAngleY = kalmanY.getAngle(accYangle, gyroYrate, (double)(micros() - timer) / 1000000);
  timer = micros();
}
void i2cWrite(uint8_t IMUAddress, uint8_t registerAddress, uint8_t data) {
  Wire.beginTransmission(IMUAddress);
  Wire.write(registerAddress);
  Wire.write(data);
  Wire.endTransmission();
}
uint8_t* i2cRead(uint8_t IMUAddress, uint8_t registerAddress, uint8_t nbytes) {
  uint8_t data[nbytes];
  Wire.beginTransmission(IMUAddress);
  Wire.write(registerAddress);
  Wire.endTransmission(false);
  Wire.requestFrom(IMUAddress, nbytes);
  for (uint8_t i = 0; i < nbytes; i++)
    data [i] = Wire.read();
  return data;
}